import { InvalidChangeError, MissingEntityError, MissingPropertyError, } from '@crosslab/service-common';
import { logger } from '@crosslab/service-common';
import { repositories } from '../../database/dataSource.js';
import { InvalidStateError, MalformedExperimentError } from '../../types/errors.js';
import { validateExperimentStatus } from '../../types/typeguards.js';
import { experimentUrlFromId } from '../url.js';
import { bookExperiment } from './book.js';
import { setupExperiment } from './setup/index.js';
/**
 * This function attempts to run an experiment.
 * @param experimentModel The experiment to be run.
 * @throws {InvalidStateError} Thrown when the status of the experiment is already "finished".
 */
export async function runExperiment(experimentModel, clients) {
    const experimentUrl = experimentUrlFromId(experimentModel.uuid);
    logger.log('info', 'Attempting to run experiment', { data: { experimentUrl } });
    // make sure experiment is not already finished
    if (experimentModel.status === 'finished') {
        throw new InvalidStateError(`Experiment status is already "finished"`, 400);
    }
    // make sure the experiment contains devices
    if (!experimentModel.devices || experimentModel.devices.length === 0) {
        throw new MissingPropertyError(`Experiment does not contain any devices`, 400);
    }
    // book experiment if status is "created"
    if (experimentModel.status === 'created') {
        await bookExperiment(experimentModel);
    }
    // check if booking times have changed and throw error because currently times may not be changed
    if (experimentModel.status === 'booked') {
        if (!validateExperimentStatus(experimentModel, 'booked'))
            throw new MalformedExperimentError(`Experiment is in status 'booked', but does not satisfy the requirements for this status`, 500);
        const booking = await clients.booking.frontend.getBooking(experimentModel.bookingID);
        if (Date.parse(experimentModel.bookingStart) !==
            Date.parse(booking.Booking.Time.Start) ||
            Date.parse(experimentModel.bookingEnd) !== Date.parse(booking.Booking.Time.End)) {
            throw new InvalidChangeError(`The start and end of a booking cannot be changed!`, 400);
        }
    }
    const resolvedDevices = await Promise.all(experimentModel.devices.map(async (device) => {
        try {
            const resolvedDevice = await clients.device.getDevice(device.url);
            return {
                ...resolvedDevice,
                instanceUrl: device.instance?.url,
                instanceToken: device.instance?.token,
            };
        }
        catch (error) {
            console.error(error);
            throw new MissingEntityError(`Device "${device.url}" could not be resolved`, 404);
        }
    }));
    await setupExperiment(experimentModel, resolvedDevices, clients);
    // save experiment
    await repositories.experiment.save(experimentModel);
    logger.log('info', 'Successfully running experiment', { data: { experimentUrl } });
}
//# sourceMappingURL=run.js.map